using System.Collections;
using System.Collections.Generic;
using System.Xml.Serialization;
using UnityEngine;

public class LDAIDoActionComponent : MonoBehaviour
{

    protected GameManager theGame;
    protected Vector2 vGoalLocation;
    public LDGameActor myOwningActor;
    public ECreatureAIState myStateIfActive;


    public virtual void Awake()
    {
        theGame = GameManager.GetGame();
    }

	// Use this for initialization
    public virtual void Start()
    {
		
	}
	
	// Update is called once per frame
	public virtual void Update () 
    {
		
	}

    public virtual Vector2 WhereShouldIGo()
    {
        return vGoalLocation;
    }

    public virtual bool FigureOutWhereToGo()
    {
        return true;
    }

    public virtual ECreatureAIState TakeAction(ECreatureAIState myState)
    {
        return myState;
    }

    public virtual string GetThoughtText()
    {
        return vGoalLocation.ToString();
    }

    public virtual void OnRegisterAI()
    {
        
    }

    public virtual LDGameActor GetTargetActor()
    {
        return null;
    }

    public virtual void SetAnchorActor(LDGameActor anchor, float leash = 0f)
    {
        
    }
}
